@@ -4,9 +4,11 @@ import logging
 import os
 import threading
 from functools import partial
+from collections import namedtuple
 
 import requests
 import six
+import humanize
 from more_executors import Executors
 from more_executors.futures import f_map, f_flat_map, f_return, f_proxy, f_sequence
 from six.moves import StringIO
@@ -21,6 +23,7 @@ from ..model import (
     Unit,
     Task,
 )
+from ..log import TimedLogger
 from .search import search_for_criteria
 from .errors import PulpException
 from .poller import TaskPoller
@@ -31,6 +34,8 @@ from .ud_mappings import compile_ud_mappings
 
 LOG = logging.getLogger("pubtools.pulplib")
 
+UploadResult = namedtuple("UploadResult", ["checksum", "size"])
+
 
 class Client(object):
     """A client for the Pulp 2.x API.
@@ -112,9 +117,10 @@ class Client(object):
     # These are not a documented/supported feature of the library.
     # They're more of an emergency escape hatch to temporarily resolve issues.
     _REQUEST_THREADS = int(os.environ.get("PUBTOOLS_PULPLIB_REQUEST_THREADS", "4"))
+    _UPLOAD_THREADS = int(os.environ.get("PUBTOOLS_PULPLIB_UPLOAD_THREADS", "4"))
     _PAGE_SIZE = int(os.environ.get("PUBTOOLS_PULPLIB_PAGE_SIZE", "2000"))
     _TASK_THROTTLE = int(os.environ.get("PUBTOOLS_PULPLIB_TASK_THROTTLE", "200"))
-    _CHUNK_SIZE = int(os.environ.get("PUBTOOLS_PULPLIB_CHUNK_SIZE", 1024 * 1024))
+    _CHUNK_SIZE = int(os.environ.get("PUBTOOLS_PULPLIB_CHUNK_SIZE", 1024 * 1024 * 10))
 
     # Policy used when deciding whether to retry operations.
     # This is mainly provided here as a hook for autotests, so the policy can be
@@ -181,6 +187,15 @@ class Client(object):
             .with_retry(retry_policy=self._RETRY_POLICY)
         )
 
+        # executor for uploads:
+        # - reading of file content for upload, and calculation of checksums,
+        #   happens in this executor
+        # - HTTP requests don't happen here and are still submitted via
+        #   request_executor (hence no retry needed on this executor)
+        self._upload_executor = Executors.thread_pool(
+            name="pubtools-pulplib-uploads", max_workers=self._UPLOAD_THREADS
+        )
+
         # executor for issuing HTTP requests to Pulp which spawn tasks:
         # - checks HTTP response and raises if it's not JSON or
         #   it's not successful
@@ -205,6 +220,7 @@ class Client(object):
 
     def __exit__(self, *args, **kwargs):
         self._request_executor.__exit__(*args, **kwargs)
+        self._upload_executor.__exit__(*args, **kwargs)
         self._task_executor.__exit__(*args, **kwargs)
 
     def get_repository(self, repository_id):
@@ -536,30 +552,84 @@ class Client(object):
         # We only support returning the ID at this time.
         return f_proxy(f_map(out, lambda types: sorted([t["id"] for t in types])))
 
-    def _do_upload_file(self, upload_id, file_obj):
-        def do_next_upload(checksum, size):
-            data = file_obj.read(self._CHUNK_SIZE)
-            if data:
-                if isinstance(data, six.text_type):
-                    # if it's unicode, need to encode before calculate checksum
-                    data = data.encode("utf-8")
-                checksum.update(data)
-                return f_flat_map(
-                    self._do_upload(data, upload_id, size),
-                    lambda _: do_next_upload(checksum, size + len(data)),
-                )
-            # nothing more to upload, return checksum and size
-            return f_return((checksum.hexdigest(), size))
+    def _do_upload_file(self, upload_id, file_obj, name="<unknown file>"):
+        return self._upload_executor.submit(
+            self._upload_file_loop, upload_id, file_obj, name
+        )
+
+    def _upload_file_loop(self, upload_id, file_obj, name):
+        # Read a file in chunks, upload it to pulp under the given upload_id,
+        # and return the checksum & bytes read.
+        #
+        # This method will block for the entire duration of the upload, which can
+        # be slow for a large file. It is intended to be invoked only from within
+        # the upload_executor in order to avoid blocking other operations.
+
+        total_size = None
 
         is_file_object = "close" in dir(file_obj)
         if not is_file_object:
+            # This is the preferred case (we're responsible for opening the file),
+            # as we can then know the total expected size. (file-like objects in
+            # general do not know their own size)
+            total_size = os.path.getsize(file_obj)
             file_obj = open(file_obj, "rb")
 
-        upload_f = f_flat_map(f_return(), lambda _: do_next_upload(hashlib.sha256(), 0))
+        upload_logger = TimedLogger()
+        checksum = hashlib.sha256()
+        size = 0
 
-        upload_f.add_done_callback(lambda _: file_obj.close())
+        # We allow having a few chunks in flight at once, up to as many requests
+        # as we can do in parallel.
+        prev_chunks = [f_return() for _ in range(0, self._REQUEST_THREADS)]
 
-        return upload_f
+        try:
+            while True:
+                data = file_obj.read(self._CHUNK_SIZE)
+
+                if not data:
+                    break
+
+                if isinstance(data, six.text_type):
+                    # if it's unicode, need to encode before calculate checksum
+                    data = data.encode("utf-8")
+
+                checksum.update(data)
+
+                # Ensure the number of chunks are kept at a fixed size by waiting
+                # for one of them to complete.
+                prev_chunks.pop(0).result()
+
+                # Then start upload of this chunk to Pulp.
+                prev_chunks.append(self._do_upload(data, upload_id, size))
+
+                size += len(data)
+
+                # Log a message about the upload progress.
+                if not total_size:
+                    # no percentage can be calculated
+                    pct = ""
+                else:
+                    pct = float(size) / total_size * 100
+                    pct = " / %2d%%" % pct
+
+                upload_logger.info(
+                    "Still uploading %s: %s%s [%s]",
+                    name,
+                    humanize.naturalsize(size),
+                    pct,
+                    upload_id,
+                )
+
+            # No more data to read. As soon as all chunks in progress are done,
+            # we have finished with the upload.
+            for chunk in prev_chunks:
+                chunk.result()
+
+            return UploadResult(checksum.hexdigest(), size)
+
+        finally:
+            file_obj.close()
 
     def _publish_repository(self, repo, distributors_with_config):
         compiled = self._compile_notes(repo)
@@ -762,10 +832,10 @@ class Client(object):
         LOG.debug("Queuing request to DELETE %s", url)
         return self._task_executor.submit(self._do_request, method="DELETE", url=url)
 
-    def _request_upload(self):
+    def _request_upload(self, name):
         url = os.path.join(self._url, "pulp/api/v2/content/uploads/")
 
-        LOG.debug("Requesting upload id")
+        LOG.debug("Requesting upload id for %s", name)
         return self._request_executor.submit(self._do_request, method="POST", url=url)
 
     def _do_upload(self, data, upload_id, offset):
@@ -798,10 +868,10 @@ class Client(object):
             self._do_request, method="POST", url=url, json=body
         )
 
-    def _delete_upload_request(self, upload_id):
+    def _delete_upload_request(self, upload_id, name):
         url = os.path.join(self._url, "pulp/api/v2/content/uploads/%s/" % upload_id)
 
-        LOG.debug("Deleting upload request %s", upload_id)
+        LOG.debug("Deleting upload request %s for %s", upload_id, name)
         return self._request_executor.submit(self._do_request, method="DELETE", url=url)
 
     def _do_get_maintenance(self):
